@@ -1,6 +1,6 @@
 /* TUI Interpreter definitions for GDB, the GNU debugger.
 
-   Copyright (C) 2003-2017 Free Software Foundation, Inc.
+   Copyright (C) 2003-2018 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -31,7 +31,7 @@
 #include "tui/tui.h"
 #include "tui/tui-io.h"
 #include "infrun.h"
-#include "observer.h"
+#include "observable.h"
 #include "gdbthread.h"
 
 /* Set to 1 when the TUI mode must be activated when we first start
@@ -58,9 +58,7 @@ public:
 static tui_interp *
 as_tui_interp (struct interp *interp)
 {
-  if (strcmp (interp_name (interp), INTERP_TUI) == 0)
-    return (tui_interp *) interp;
-  return NULL;
+  return dynamic_cast<tui_interp *> (interp);
 }
 
 /* Cleanup the tui before exiting.  */
@@ -326,14 +324,14 @@ _initialize_tui_interp (void)
     }
 
   /* If changing this, remember to update cli-interp.c as well.  */
-  observer_attach_normal_stop (tui_on_normal_stop);
-  observer_attach_signal_received (tui_on_signal_received);
-  observer_attach_end_stepping_range (tui_on_end_stepping_range);
-  observer_attach_signal_exited (tui_on_signal_exited);
-  observer_attach_exited (tui_on_exited);
-  observer_attach_no_history (tui_on_no_history);
-  observer_attach_sync_execution_done (tui_on_sync_execution_done);
-  observer_attach_command_error (tui_on_command_error);
-  observer_attach_user_selected_context_changed
+  gdb::observers::normal_stop.attach (tui_on_normal_stop);
+  gdb::observers::signal_received.attach (tui_on_signal_received);
+  gdb::observers::end_stepping_range.attach (tui_on_end_stepping_range);
+  gdb::observers::signal_exited.attach (tui_on_signal_exited);
+  gdb::observers::exited.attach (tui_on_exited);
+  gdb::observers::no_history.attach (tui_on_no_history);
+  gdb::observers::sync_execution_done.attach (tui_on_sync_execution_done);
+  gdb::observers::command_error.attach (tui_on_command_error);
+  gdb::observers::user_selected_context_changed.attach
     (tui_on_user_selected_context_changed);
 }
